@@ -7,25 +7,30 @@
    installed, parameters can be constrained as a simple
    mathematical expression of other Parameters.
 
-   version: 0.7.2
-   last update: 20-June-2013
+   version: 0.7.4
+   last update: 2014-Jan-22
    License: BSD
    Author:  Matthew Newville <newville@cars.uchicago.edu>
             Center for Advanced Radiation Sources,
             The University of Chicago
 """
-__version__ = '0.7.2'
-from .minimizer import minimize, Minimizer, MinimizerException, make_paras_and_func
+__version__ = '0.7.4'
+from .minimizer import minimize, Minimizer, MinimizerException
 from .parameter import Parameter, Parameters
 from .confidence import conf_interval, conf_interval2d
 from .printfuncs import (fit_report, ci_report,
                          report_fit, report_ci, report_errors)
 
+from .wrap      import wrap_function, make_paras_and_func
+from .model import Model
+from .specified_models import *
+
+from . import models1d
 from . import uncertainties
 from .uncertainties import ufloat, correlated_values
 
 __xall__ = ['minimize', 'Minimizer', 'Parameter', 'Parameters',
-           'conf_interval', 'conf_interval2d', 'make_paras_and_func',
-           'fit_report', 'ci_report', 'report_errors',
-           'report_fit', 'report_ci', 'ufloat',
-           'correlated_values']
+            'conf_interval', 'conf_interval2d', 'wrap_function',
+            'make_paras_and_func', 'fit_report', 'ci_report',
+            'report_errors', 'report_fit', 'report_ci', 'ufloat',
+            'correlated_values']
