@@ -9,15 +9,15 @@ dictionary supporting a simple, flat namespace.
 Expressions can be compiled into ast node and then evaluated
 later, using the current values in the
 """
+
 from __future__ import division, print_function
 from sys import exc_info, stdout, version_info
 import ast
 import math
-from .astutils import (FROM_PY, FROM_MATH, FROM_NUMPY,
-                       NUMPY_RENAMES, ExceptionHolder)
-
-from .parameter import isParameter, valid_symbol_name
 
+from .astutils import (FROM_PY, FROM_MATH, FROM_NUMPY, UNSAFE_ATTRS,
+                       NUMPY_RENAMES, op2func, ExceptionHolder,
+                       ReturnedNone, valid_symbol_name)
 HAS_NUMPY = False
 try:
     import numpy
@@ -25,50 +25,8 @@ try:
 except ImportError:
     print("Warning: numpy not available... functionality will be limited.")
 
+__version__ = '0.3'
 
-OPERATORS = {
-    ast.Add:    lambda a, b: b.__radd__(a) if isParameter(b) else a + b,
-    ast.Sub:    lambda a, b: b.__rsub__(a) if isParameter(b) else a - b,
-    ast.Mult:   lambda a, b: b.__rmul__(a) if isParameter(b) else a * b,
-    ast.Div:    lambda a, b: b.__rdiv__(a) if isParameter(b) else a / b,
-    ast.FloorDiv: lambda a, b: b.__rfloordiv__(a) if isParameter(b) else a // b,
-    ast.Mod:    lambda a, b: b.__rmod__(a) if isParameter(b) else a % b,
-    ast.Pow:    lambda a, b: b.__rpow__(a) if isParameter(b) else a ** b,
-    ast.Eq:     lambda a, b: b.__eq__(a)  if isParameter(b) else a == b,
-    ast.Gt:     lambda a, b: b.__le__(a) if isParameter(b) else a > b,
-    ast.GtE:    lambda a, b: b.__lt__(a) if isParameter(b) else a >= b,
-    ast.Lt:     lambda a, b: b.__ge__(a) if isParameter(b) else a < b,
-    ast.LtE:    lambda a, b: b.__gt__(a) if isParameter(b) else a <= b,
-    ast.NotEq:  lambda a, b: b.__ne__(a) if isParameter(b) else a != b,
-    ast.Is:     lambda a, b: a is b,
-    ast.IsNot:  lambda a, b: a is not b,
-    ast.In:     lambda a, b: a in b,
-    ast.NotIn:  lambda a, b: a not in b,
-    ast.BitAnd: lambda a, b: a & b,
-    ast.BitOr:  lambda a, b: a | b,
-    ast.BitXor: lambda a, b: a ^ b,
-    ast.LShift: lambda a, b: a << b,
-    ast.RShift: lambda a, b: a >> b,
-    ast.And:    lambda a, b: a and b,
-    ast.Or:     lambda a, b: a or b,
-    ast.Invert: lambda a: ~a,
-    ast.Not:    lambda a: not a,
-    ast.UAdd:   lambda a: +a,
-    ast.USub:   lambda a: -a}
-
-def op2func(op):
-    "return function for operator nodes"
-    return OPERATORS[op.__class__]
-
-__version__ = '0.3.1'
-
-# holder for 'returned None' from Larch procedure
-class Empty:
-    """basic empty containter"""
-    def __nonzero__(self):
-        return False
-
-ReturnedNone = Empty()
 
 class Interpreter:
     """mathematical expression compiler and interpreter.
@@ -111,13 +69,12 @@ class Interpreter:
 
         if symtable is None:
             symtable = {}
-        self.symtable   = symtable
+        self.symtable = symtable
         self._interrupt = None
-        self.error      = []
-        self.expr       = None
-        self.retval     = None
-        self.errmsg =  None
-        self.lineno    = 0
+        self.error = []
+        self.expr = None
+        self.retval = None
+        self.lineno = 0
         global HAS_NUMPY
         if not use_numpy:
             HAS_NUMPY = False
@@ -144,22 +101,19 @@ class Interpreter:
     def unimplemented(self, node):
         "unimplemented nodes"
         self.raise_exception(node, exc=NotImplementedError,
-             msg="'%s' not supported" % (node.__class__.__name__))
+                             msg="'%s' not supported" %
+                             (node.__class__.__name__))
 
     def raise_exception(self, node, exc=None, msg='', expr=None,
                         lineno=None):
         "add an exception"
         if self.error is None:
             self.error = []
-        if expr  is None:
-            expr  = self.expr
-
+        if expr is None:
+            expr = self.expr
         if len(self.error) > 0 and not isinstance(node, ast.Module):
             msg = '%s' % msg
-        if self.errmsg is None:
-            self.errmsg = msg
-        err = ExceptionHolder(node, exc=exc, msg=self.errmsg,
-                              expr=expr, lineno=lineno)
+        err = ExceptionHolder(node, exc=exc, msg=msg, expr=expr, lineno=lineno)
         self._interrupt = ast.Break()
         self.error.append(err)
         raise RuntimeError(err.msg)
@@ -170,7 +124,7 @@ class Interpreter:
     #  eval:   string statement -> result = run(parse(statement))
     def parse(self, text):
         """parse statement/expression to Ast representation"""
-        self.expr  = text
+        self.expr = text
         try:
             return ast.parse(text)
         except:
@@ -181,7 +135,7 @@ class Interpreter:
         """executes parsed Ast representation for an expression"""
         # Note: keep the 'node is None' test: internal code here may run
         #    run(None) and expect a None in return.
-        if len(self.error)>0:
+        if len(self.error) > 0:
             return
         if node is None:
             return None
@@ -189,9 +143,8 @@ class Interpreter:
             node = self.parse(node)
         if lineno is not None:
             self.lineno = lineno
-
-        if expr   is not None:
-            self.expr   = expr
+        if expr is not None:
+            self.expr = expr
 
         # get handler for this node:
         #   on_xxx with handle nodes of type 'xxx', etc
@@ -217,7 +170,6 @@ class Interpreter:
     def eval(self, expr, lineno=0, show_errors=True):
         """evaluates a single statement"""
         self.lineno = lineno
-        self.errmsg =  None
         self.error = []
         try:
             node = self.parse(expr)
@@ -253,7 +205,7 @@ class Interpreter:
         "index"
         return self.run(node.value)  # ('value',)
 
-    def on_return(self, node): # ('value',)
+    def on_return(self, node):  # ('value',)
         "return statement: look for None, return special sentinal"
         self.retval = self.run(node.value)
         if self.retval is None:
@@ -309,14 +261,14 @@ class Interpreter:
 
     def on_dict(self, node):    # ('keys', 'values')
         "dictionary"
-        return dict([(self.run(k), self.run(v)) for k, v in \
+        return dict([(self.run(k), self.run(v)) for k, v in
                      zip(node.keys, node.values)])
 
-    def on_num(self, node): # ('n',)
+    def on_num(self, node):   # ('n',)
         'return number'
         return node.n
 
-    def on_str(self, node): # ('s',)
+    def on_str(self, node):   # ('s',)
         'return string'
         return node.s
 
@@ -332,7 +284,6 @@ class Interpreter:
                 msg = "name '%s' is not defined" % node.id
                 self.raise_exception(node, exc=NameError, msg=msg)
 
-
     def node_assign(self, node, val):
         """here we assign a value (not the node.value object) to a node
         this is used by on_assign, but also by for, list comprehension, etc.
@@ -343,14 +294,14 @@ class Interpreter:
                 self.raise_exception(node, exc=NameError, msg=errmsg)
             sym = self.symtable[node.id] = val
         elif node.__class__ == ast.Attribute:
-            if node.ctx.__class__  == ast.Load:
+            if node.ctx.__class__ == ast.Load:
                 msg = "cannot assign to attribute %s" % node.attr
                 self.raise_exception(node, exc=AttributeError, msg=msg)
 
             setattr(self.run(node.value), node.attr, val)
 
         elif node.__class__ == ast.Subscript:
-            sym    = self.run(node.value)
+            sym = self.run(node.value)
             xslice = self.run(node.slice)
             if isinstance(node.slice, ast.Index):
                 sym[xslice] = val
@@ -370,7 +321,7 @@ class Interpreter:
         ctx = node.ctx.__class__
         if ctx == ast.Load:
             sym = self.run(node.value)
-            if hasattr(sym, node.attr):
+            if hasattr(sym, node.attr) and node.attr not in UNSAFE_ATTRS:
                 return getattr(sym, node.attr)
             else:
                 obj = self.run(node.value)
@@ -389,14 +340,14 @@ class Interpreter:
         val = self.run(node.value)
         for tnode in node.targets:
             self.node_assign(tnode, val)
-        return # return val
+        return
 
     def on_augassign(self, node):    # ('target', 'op', 'value')
         "augmented assign"
         return self.on_assign(ast.Assign(targets=[node.target],
-                                         value=ast.BinOp(left = node.target,
-                                                         op = node.op,
-                                                         right= node.value)))
+                                         value=ast.BinOp(left=node.target,
+                                                         op=node.op,
+                                                         right=node.value)))
 
     def on_slice(self, node):    # ():('lower', 'upper', 'step')
         "simple slice"
@@ -404,17 +355,16 @@ class Interpreter:
                      self.run(node.upper),
                      self.run(node.step))
 
-
     def on_extslice(self, node):    # ():('dims',)
         "extended slice"
         return tuple([self.run(tnode) for tnode in node.dims])
 
     def on_subscript(self, node):    # ('value', 'slice', 'ctx')
         "subscript handling -- one of the tricky parts"
-        val    = self.run(node.value)
+        val = self.run(node.value)
         nslice = self.run(node.slice)
         ctx = node.ctx.__class__
-        if ctx in ( ast.Load, ast.Store):
+        if ctx in (ast.Load, ast.Store):
             if isinstance(node.slice, (ast.Index, ast.Slice, ast.Ellipsis)):
                 return val.__getitem__(nslice)
             elif isinstance(node.slice, ast.ExtSlice):
@@ -464,10 +414,10 @@ class Interpreter:
     def on_compare(self, node):    # ('left', 'ops', 'comparators')
         "comparison operators"
         lval = self.run(node.left)
-        out  = True
+        out = True
         for op, rnode in zip(node.ops, node.comparators):
             rval = self.run(rnode)
-            out  = op2func(op)(lval, rval)
+            out = op2func(op)(lval, rval)
             lval = rval
             if HAS_NUMPY and isinstance(out, numpy.ndarray) and out.any():
                 break
@@ -483,7 +433,7 @@ class Interpreter:
         if node.nl:
             end = '\n'
         out = [self.run(tnode) for tnode in node.values]
-        if out and len(self.error)==0:
+        if out and len(self.error) == 0:
             self._printer(*out, file=dest, end=end)
 
     def _printer(self, *out, **kws):
@@ -557,7 +507,7 @@ class Interpreter:
                         out.append(self.run(node.elt))
         return out
 
-    def on_excepthandler(self, node): # ('type', 'name', 'body')
+    def on_excepthandler(self, node):  # ('type', 'name', 'body')
         "exception handler..."
         return (self.run(node.type), node.name, node.body)
 
@@ -587,12 +537,12 @@ class Interpreter:
     def on_raise(self, node):    # ('type', 'inst', 'tback')
         "raise statement: note difference for python 2 and 3"
         if version_info[0] == 3:
-            excnode  = node.exc
-            msgnode  = node.cause
+            excnode = node.exc
+            msgnode = node.cause
         else:
-            excnode  = node.type
-            msgnode  = node.inst
-        out  = self.run(excnode)
+            excnode = node.type
+            msgnode = node.inst
+        out = self.run(excnode)
         msg = ' '.join(out.args)
         msg2 = self.run(msgnode)
         if msg2 not in (None, 'None'):
@@ -625,7 +575,7 @@ class Interpreter:
             return func(*args, **keywords)
         except:
             self.raise_exception(node, exc=RuntimeError,
-                                 msg = "Error running %s" % (func))
+                                 msg="Error running %s" % (func))
 
     def on_arg(self, node):    # ('test', 'msg')
         "arg for function definitions"
@@ -651,18 +601,17 @@ class Interpreter:
             args = [tnode.id for tnode in node.args.args[:offset]]
 
         doc = None
-        if (isinstance(node.body[0], ast.Expr) and
-            isinstance(node.body[0].value, ast.Str)):
-            docnode = node.body[0]
-            doc = docnode.value.s
+        nb0 = node.body[0]
+        if isinstance(nb0, ast.Expr) and isinstance(nb0.value, ast.Str):
+            doc = nb0.value.s
 
         self.symtable[node.name] = Procedure(node.name, self, doc=doc,
-                                             body = node.body,
-                                             lineno = self.lineno,
-                                             args = args,
-                                             kwargs = kwargs,
-                                             vararg = node.args.vararg,
-                                             varkws = node.args.kwarg)
+                                             lineno=self.lineno,
+                                             body=node.body,
+                                             args=args, kwargs=kwargs,
+                                             vararg=node.args.vararg,
+                                             varkws=node.args.kwarg)
+
 
 class Procedure(object):
     """Procedure: user-defined function for asteval
@@ -731,10 +680,9 @@ class Procedure(object):
             msg = None
             if n_args < n_names:
                 msg = 'not enough arguments for Procedure %s()' % self.name
-                msg = '%s (expected %i, got %i)'% (msg, n_names, n_args)
+                msg = '%s (expected %i, got %i)' % (msg, n_names, n_args)
                 self.raise_exc(None, exc=TypeError, msg=msg)
 
-
         for argname in self.argnames:
             symlocals[argname] = args.pop(0)
 
@@ -773,7 +721,8 @@ class Procedure(object):
                 break
             if self.interpreter.retval is not None:
                 retval = self.interpreter.retval
-                if retval is ReturnedNone: retval = None
+                if retval is ReturnedNone:
+                    retval = None
                 break
 
         self.interpreter.symtable = save_symtable
