@@ -11,6 +11,8 @@ function-to-be-minimized (residual function) in terms of these Parameters.
    <newville@cars.uchicago.edu>
 """
 
+from copy import deepcopy
+import numpy as np
 from numpy import (dot, eye, ndarray, ones_like,
                    sqrt, take, transpose, triu)
 from numpy.dual import inv
@@ -36,6 +38,7 @@ from .parameter import Parameter, Parameters
 # use locally modified version of uncertainties package
 from . import uncertainties
 
+
 def asteval_with_uncertainties(*vals,  **kwargs):
     """
     given values for variables, calculate object value.
@@ -43,8 +46,8 @@ def asteval_with_uncertainties(*vals,  **kwargs):
     the uncertainty in an object even with a complicated
     expression.
     """
-    _obj   = kwargs.get('_obj', None)
-    _pars  = kwargs.get('_pars', None)
+    _obj = kwargs.get('_obj', None)
+    _pars = kwargs.get('_pars', None)
     _names = kwargs.get('_names', None)
     _asteval = kwargs.get('_asteval', None)
     if (_obj is None or
@@ -59,6 +62,7 @@ def asteval_with_uncertainties(*vals,  **kwargs):
 
 wrap_ueval = uncertainties.wrap(asteval_with_uncertainties)
 
+
 def eval_stderr(obj, uvars, _names, _pars, _asteval):
     """evaluate uncertainty and set .stderr for a parameter `obj`
     given the uncertain values `uvars` (a list of uncertainties.ufloats),
@@ -77,6 +81,7 @@ def eval_stderr(obj, uvars, _names, _pars, _asteval):
     except:
         obj.stderr = 0
 
+
 class MinimizerException(Exception):
     """General Purpose Exception"""
     def __init__(self, msg):
@@ -86,6 +91,7 @@ class MinimizerException(Exception):
     def __str__(self):
         return "\n%s" % (self.msg)
 
+
 def check_ast_errors(error):
     """check for errors derived from asteval, raise MinimizerException"""
     if len(error) > 0:
@@ -99,7 +105,7 @@ class Minimizer(object):
     """general minimizer"""
     err_nonparam = \
      "params must be a minimizer.Parameters() instance or list of Parameters()"
-    err_maxfev   = """Too many function calls (max set to  %i)!  Use:
+    err_maxfev = """Too many function calls (max set to  %i)!  Use:
     minimize(func, params, ...., maxfev=NNN)
 or set  leastsq_kws['maxfev']  to increase this maximum."""
 
@@ -125,7 +131,12 @@ or set  leastsq_kws['maxfev']  to increase this maximum."""
         self.namefinder = NameFinder()
         self.__prepared = False
         self.__set_params(params)
-        self.prepare_fit()
+        # self.prepare_fit()
+
+    @property
+    def values(self):
+        "Convenience function that returns Parameter values as a simple dict."
+        return dict([(name, p.value) for name, p in self.params.items()])
 
     def __update_paramval(self, name):
         """
@@ -140,9 +151,12 @@ or set  leastsq_kws['maxfev']  to increase this maximum."""
         if self.updated[name]:
             return
         par = self.params[name]
-        if par.expr is not None:
-            for dep in par.deps:
-                self.__update_paramval(dep)
+        if getattr(par, 'expr', None) is not None:
+            if getattr(par, 'ast', None) is None:
+                par.ast = self.asteval.parse(par.expr)
+            if par.deps is not None:
+                for dep in par.deps:
+                    self.__update_paramval(dep)
             par.value = self.asteval.run(par.ast)
             out = check_ast_errors(self.asteval.error)
             if out is not None:
@@ -187,7 +201,7 @@ or set  leastsq_kws['maxfev']  to increase this maximum."""
         """
         for varname, val in zip(self.var_map, fvars):
             # self.params[varname].value = val
-            self.params[varname].from_internal(val)
+            self.params[varname].value = self.params[varname].from_internal(val)
 
         self.nfev = self.nfev + 1
         self.update_constraints()
@@ -264,6 +278,18 @@ or set  leastsq_kws['maxfev']  to increase this maximum."""
         self.update_constraints()
         self.__prepared = True
 
+    def unprepare_fit(self):
+        """unprepare fit, so that subsequent fits will be
+        forced to run re-prepare the fit
+
+        removes ast compilations of constraint expressions
+        """
+        self.__prepared = False
+        self.params = deepcopy(self.params)
+        for par in self.params.values():
+            if hasattr(par, 'ast'):
+                delattr(par, 'ast')
+
     def anneal(self, schedule='cauchy', **kws):
         """
         use simulated annealing
@@ -274,13 +300,14 @@ or set  leastsq_kws['maxfev']  to increase this maximum."""
 
         self.prepare_fit()
         sakws = dict(full_output=1, schedule=sched,
-                     maxiter = 2000 * (self.nvarys + 1))
+                     maxiter=2000 * (self.nvarys + 1))
 
         sakws.update(self.kws)
         sakws.update(kws)
         print("WARNING:  scipy anneal appears unusable!")
         saout = scipy_anneal(self.penalty, self.vars, **sakws)
         self.sa_out = saout
+        self.unprepare_fit()
         return
 
     def lbfgsb(self, **kws):
@@ -289,16 +316,25 @@ or set  leastsq_kws['maxfev']  to increase this maximum."""
         """
         self.prepare_fit()
         lb_kws = dict(factr=1000.0, approx_grad=True, m=20,
-                      maxfun = 2000 * (self.nvarys + 1),
+                      maxfun=2000 * (self.nvarys + 1),
                       # bounds = zip(self.vmin, self.vmax),
                       )
         lb_kws.update(self.kws)
         lb_kws.update(kws)
 
         xout, fout, info = scipy_lbfgsb(self.penalty, self.vars, **lb_kws)
-        self.nfev =  info['funcalls']
+        self.nfev = info['funcalls']
         self.message = info['task']
-        self.chisqr = (self.penalty(xout)**2).sum()
+        self.chisqr = self.residual = self.__residual(xout)
+        self.ndata = 1
+        self.nfree = 1
+        if isinstance(self.residual, ndarray):
+            self.chisqr = (self.chisqr**2).sum()
+            self.ndata = len(self.residual)
+            self.nfree = self.ndata - self.nvarys
+        self.redchi = self.chisqr/self.nfree
+        self.unprepare_fit()
+        return
 
     def fmin(self, **kws):
         """
@@ -307,15 +343,24 @@ or set  leastsq_kws['maxfev']  to increase this maximum."""
         self.prepare_fit()
         fmin_kws = dict(full_output=True, disp=False, retall=True,
                         ftol=1.e-4, xtol=1.e-4,
-                        maxfun = 5000 * (self.nvarys + 1))
+                        maxfun=5000 * (self.nvarys + 1))
 
         fmin_kws.update(kws)
         ret = scipy_fmin(self.penalty, self.vars, **fmin_kws)
         xout, fout, iter, funccalls, warnflag, allvecs = ret
-        self.nfev =  funccalls
-        self.chisqr = (self.penalty(xout)**2).sum()
+        self.nfev = funccalls
+        self.chisqr = self.residual = self.__residual(xout)
+        self.ndata = 1
+        self.nfree = 1
+        if isinstance(self.residual, ndarray):
+            self.chisqr = (self.chisqr**2).sum()
+            self.ndata = len(self.residual)
+            self.nfree = self.ndata - self.nvarys
+        self.redchi = self.chisqr/self.nfree
+        self.unprepare_fit()
+        return
 
-    def scalar_minimize(self, method='Nelder-Mead', hess=None, tol=None, **kws):
+    def scalar_minimize(self, method='Nelder-Mead', **kws):
         """use one of the scaler minimization methods from scipy.
         Available methods include:
           Nelder-Mead
@@ -328,6 +373,8 @@ or set  leastsq_kws['maxfev']  to increase this maximum."""
           TNC
           COBYLA
           SLSQP
+          dogleg
+          trust-ncg
 
         If the objective function returns a numpy array instead
         of the expected scalar, the sum of squares of the array
@@ -338,25 +385,45 @@ or set  leastsq_kws['maxfev']  to increase this maximum."""
         for those designed to use bounds.
 
         """
-        if not HAS_SCALAR_MIN :
+        if not HAS_SCALAR_MIN:
             raise NotImplementedError
 
         self.prepare_fit()
 
-        maxfev = 1000*(self.nvarys + 1)
-        opts = {'maxiter': maxfev}
-        if method not in ('L-BFGS-B', 'TNC', 'SLSQP'):
-            opts['maxfev'] = maxfev
-
-        fmin_kws = dict(method=method, tol=tol, hess=hess, options=opts)
+        fmin_kws = dict(method=method,
+                        options={'maxiter': 1000*(self.nvarys + 1)})
         fmin_kws.update(self.kws)
         fmin_kws.update(kws)
 
+        # hess supported only in some methods
+        if 'hess' in fmin_kws and method not in ('Newton-CG',
+                                                 'dogleg', 'trust-ncg'):
+            fmin_kws.pop('hess')
+
+        # jac supported only in some methods (and Dfun could be used...)
+        if 'jac' not in fmin_kws and fmin_kws.get('Dfun', None) is not None:
+            self.jacfcn = fmin_kws.pop('jac')
+            fmin_kws['jac'] = self.__jacobian
+
+        if 'jac' in fmin_kws and method not in ('CG', 'BFGS', 'Newton-CG',
+                                                'dogleg', 'trust-ncg'):
+            self.jacfcn = None
+            fmin_kws.pop('jac')
+
         ret = scipy_minimize(self.penalty, self.vars, **fmin_kws)
         xout = ret.x
         self.message = ret.message
         self.nfev = ret.nfev
-        self.chisqr = (self.penalty(xout)**2).sum()
+        self.chisqr = self.residual = self.__residual(xout)
+        self.ndata = 1
+        self.nfree = 1
+        if isinstance(self.residual, ndarray):
+            self.chisqr = (self.chisqr**2).sum()
+            self.ndata = len(self.residual)
+            self.nfree = self.ndata - self.nvarys
+        self.redchi = self.chisqr/self.nfree
+        self.unprepare_fit()
+        return
 
     def leastsq(self, **kws):
         """
@@ -384,12 +451,13 @@ or set  leastsq_kws['maxfev']  to increase this maximum."""
             self.jacfcn = lskws['Dfun']
             lskws['Dfun'] = self.__jacobian
 
+        # suppress runtime warnings during fit and error analysis
+        orig_warn_settings = np.geterr()
+        np.seterr(all='ignore')
         lsout = scipy_leastsq(self.__residual, self.vars, **lskws)
         _best, _cov, infodict, errmsg, ier = lsout
 
         self.residual = resid = infodict['fvec']
-
-
         self.ier = ier
         self.lmdif_message = errmsg
         self.message = 'Fit succeeded.'
@@ -402,7 +470,7 @@ or set  leastsq_kws['maxfev']  to increase this maximum."""
         else:
             self.message = 'Tolerance seems to be too small.'
 
-        self.nfev =  infodict['nfev']
+        self.nfev = infodict['nfev']
         self.ndata = len(resid)
 
         sum_sqr = (resid**2).sum()
@@ -414,63 +482,79 @@ or set  leastsq_kws['maxfev']  to increase this maximum."""
         # grad for the variable parameters
         grad = ones_like(_best)
         vbest = ones_like(_best)
+
+        # ensure that _best, vbest, and grad are not
+        # broken 1-element ndarrays.
+        if len(np.shape(_best)) == 0:
+            _best = np.array([_best])
+        if len(np.shape(vbest)) == 0:
+            vbest = np.array([vbest])
+        if len(np.shape(grad)) == 0:
+            grad = np.array([grad])
+
         for ivar, varname in enumerate(self.var_map):
             par = self.params[varname]
             grad[ivar] = par.scale_gradient(_best[ivar])
             vbest[ivar] = par.value
 
         # modified from JJ Helmus' leastsqbound.py
-
         infodict['fjac'] = transpose(transpose(infodict['fjac']) /
                                      take(grad, infodict['ipvt'] - 1))
         rvec = dot(triu(transpose(infodict['fjac'])[:self.nvarys, :]),
                    take(eye(self.nvarys), infodict['ipvt'] - 1, 0))
         try:
-            cov = inv(dot(transpose(rvec), rvec))
+            self.covar = inv(dot(transpose(rvec), rvec))
         except (LinAlgError, ValueError):
-            cov = None
+            self.covar = None
 
+        has_expr = False
         for par in self.params.values():
             par.stderr, par.correl = 0, None
+            has_expr = has_expr or par.expr is not None
 
-        self.covar = cov
-        if cov is None:
-            self.errorbars = False
-            self.message = '%s. Could not estimate error-bars'
-        else:
-            self.errorbars = True
+        if self.covar is not None:
             if self.scale_covar:
-                self.covar = cov = cov * sum_sqr / self.nfree
-
-            # uncertainties on constrained parameters:
-            #   get values with uncertainties (including correlations),
-            #   temporarily set Parameter values to these,
-            #   re-evaluate contrained parameters to extract stderr
-            #   and then set Parameters back to best-fit value
-            uvars = uncertainties.correlated_values(vbest, self.covar)
+                self.covar = self.covar * sum_sqr / self.nfree
 
             for ivar, varname in enumerate(self.var_map):
                 par = self.params[varname]
-                par.stderr = sqrt(cov[ivar, ivar])
+                par.stderr = sqrt(self.covar[ivar, ivar])
                 par.correl = {}
                 for jvar, varn2 in enumerate(self.var_map):
                     if jvar != ivar:
-                        par.correl[varn2] = (cov[ivar, jvar]/
-                                        (par.stderr * sqrt(cov[jvar, jvar])))
+                        par.correl[varn2] = (self.covar[ivar, jvar] /
+                             (par.stderr * sqrt(self.covar[jvar, jvar])))
 
-            for pname, par in self.params.items():
-                eval_stderr(par, uvars, self.var_map,
-                            self.params, self.asteval)
+            uvars = None
+            if has_expr:
+                # uncertainties on constrained parameters:
+                #   get values with uncertainties (including correlations),
+                #   temporarily set Parameter values to these,
+                #   re-evaluate contrained parameters to extract stderr
+                #   and then set Parameters back to best-fit value
+                try:
+                    uvars = uncertainties.correlated_values(vbest, self.covar)
+                except (LinAlgError, ValueError):
+                    uvars = None
 
-            # restore nominal values
-            for v, nam in zip(uvars, self.var_map):
-                self.asteval.symtable[nam] = v.nominal_value
+                if uvars is not None:
+                    for pname, par in self.params.items():
+                        eval_stderr(par, uvars, self.var_map,
+                                    self.params, self.asteval)
+                    # restore nominal values
+                    for v, nam in zip(uvars, self.var_map):
+                        self.asteval.symtable[nam] = v.nominal_value
 
-        for par in self.params.values():
-            if hasattr(par, 'ast'):
-                delattr(par, 'ast')
+        self.errorbars = True
+        if self.covar is None:
+            self.errorbars = False
+            self.message = '%s. Could not estimate error-bars'
+
+        np.seterr(**orig_warn_settings)
+        self.unprepare_fit()
         return self.success
 
+
 def minimize(fcn, params, method='leastsq', args=None, kws=None,
              scale_covar=True, engine=None, iter_cb=None, **fit_kws):
     """simple minimization function,
@@ -480,15 +564,24 @@ def minimize(fcn, params, method='leastsq', args=None, kws=None,
     fitter = Minimizer(fcn, params, fcn_args=args, fcn_kws=kws,
                        iter_cb=iter_cb, scale_covar=scale_covar, **fit_kws)
 
-    _scalar_methods = {'nelder': 'Nelder-Mead',     'powell': 'Powell',
-                       'cg': 'CG ',                 'bfgs': 'BFGS',
-                       'newton': 'Newton-CG',       'anneal': 'Anneal',
-                       'lbfgs': 'L-BFGS-B',         'l-bfgs': 'L-BFGS-B',
-                       'tnc': 'TNC',                'cobyla': 'COBYLA',
-                       'slsqp': 'SLSQP'}
+    _scalar_methods = {'nelder': 'Nelder-Mead',
+                       'powell': 'Powell',
+                       'cg': 'CG',
+                       'bfgs': 'BFGS',
+                       'newton': 'Newton-CG',
+                       'anneal': 'Anneal',
+                       'lbfgs': 'L-BFGS-B',
+                       'l-bfgs':'L-BFGS-B',
+                       'tnc': 'TNC',
+                       'cobyla': 'COBYLA',
+                       'slsqp': 'SLSQP',
+                       'dogleg': 'dogleg',
+                       'trust-ncg': 'trust-ncg'}
 
-    _fitmethods = {'anneal': 'anneal',               'nelder': 'fmin',
-                   'lbfgsb': 'lbfgsb',               'leastsq': 'leastsq'}
+    _fitmethods = {'anneal': 'anneal',
+                   'nelder': 'fmin',
+                   'lbfgsb': 'lbfgsb',
+                   'leastsq': 'leastsq'}
 
     if engine is not None:
         method = engine
@@ -514,49 +607,3 @@ def minimize(fcn, params, method='leastsq', args=None, kws=None,
     if fitfunction is not None:
         fitfunction(**kwargs)
     return fitter
-
-def make_paras_and_func(fcn, x0, used_kwargs=None):
-    """nach 
-    A function which takes a function a makes a parameters-dict
-    for it.
-
-    Takes the function fcn. A starting guess x0 for the
-    non kwargs paramter must be also given. If kwargs
-    are used, used_kwargs is dict were the keys are the
-    used kwarg and the values are the starting values.
-    """
-    import inspect
-    args = inspect.getargspec(fcn)
-    defaults = args[-1]
-    len_def = len(defaults) if defaults is not None else 0
-    # have_defaults = args[-len(defaults):]
-    
-    args_without_defaults = len(args[0])- len_def
-
-    if len(x0) < args_without_defaults:
-        raise ValueError( 'x0 to short')
-
-    p = Parameters()
-    for i, val in enumerate(x0):
-        p.add(args[0][i], val)
-
-    if used_kwargs:
-        for arg, val in used_kwargs.items():
-            p.add(arg, val)
-    else:
-        used_kwargs = {}
-
-    def func(para):
-        "wrapped func"
-        kwdict = {}
-        
-        for arg in used_kwargs.keys():
-            kwdict[arg] = para[arg].value
-
-        vals = [para[i].value for i in p]
-        return fcn(*vals[:len(x0)], **kwdict)
-        
-    return p, func
-
-
-
